xquery version "3.1";

(:~
 : This module provides the API endpoints for retrieving citations and
   bibliographical data on texts.
 :
 : @author Simon Sendler
 : @version 1.0
 : @since 8.1.0
 :)
module namespace kmg-citation="https://mollenhauer-edition.de/ns/kmg-util";

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";
import module namespace selection = "https://mollenhauer-edition.de/ns/conversions/selection" at "../conversions/selection.xqm";
import module namespace json = "https://mollenhauer-edition.de/ns/conversions/json" at "../conversions/json.xqm";
import module namespace bibref="https://mollenhauer-edition.de/ns/conversions/bibref" at "../conversions/bibref.xqm";
import module namespace kmg-util="http://mollenhauer-edition.de/ns/util" at "../kmg-util.xqm";
import module namespace rest="http://exquery.org/ns/restxq";
import module namespace functx="http://www.functx.com";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace output ="http://www.w3.org/2010/xslt-xquery-serialization";
declare namespace tg="http://textgrid.info/namespaces/metadata/core/2010";
declare namespace lido="http://www.lido-schema.org";

(: ~
 : This functions allows to request the citation recommendation in the bibLatex format.
 :
 : @param $id The texgridURI of the TEI object
 : @param $editionType The sprecific type of edition= "Fassung"
 : @return A plain/text object
 :)
declare
  %rest:GET
  %rest:path("api/getCitation/{$id}/{$editionType}")
  %output:media-type("text/plain")
  %output:method("text")
function kmg-citation:getCitation($id as xs:string*, $editionType as xs:string*) as xs:string* {
    let $versionToken := if ($editionType = "digital") then "sta" else $editionType
    let $biblStruct := bibref:make-biblstruct(doc($config:tei || $id || '.xml'), $versionToken)
    let $subtitle := $biblStruct/tei:analytic/tei:title[@type="sub"]/string()
    let $titleaddon :=
        if ($editionType = "commentary") then
            "Werkkommentar zu KMG " || $biblStruct//tei:idno[@type = 'editionnumber']/@kmg
        else translate(translate(bibref:format-versionAndEditionNumber($biblStruct), "(", ""), ").", "")
    let $url := "https://mollenhauer-edition.de/kmg.html?file=" || $id || "&amp;edition=" || $editionType
    return
        try {
          string-join(
                (
                    "@inbook{" || "Mollenhauer." || $id || "." || $editionType || ".2024",
                    "  author = {" || bibref:format-authors-biblatex($biblStruct/tei:analytic) || "}",
                    "  booktitle = {Klaus Mollenhauer Gesamtausgabe}",
                    "  booksubtitle = {Historisch-kritische Edition}",
                    "  editor = {Cornelie Dietrich and Klaus-Peter Horn and Hans-Rüdiger Müller}",
                    (: "  publisher = {" || "Example Publishers" || "},&#10;" || :)
                    if ($subtitle) then "  subtitle = {" || $subtitle || "}"
                    else  (),
                    "  title = {" || $biblStruct/tei:analytic/tei:title[@type="main"]/string() || "}",
                    "  titleaddon = {" || $titleaddon || "}",
                    "  year = {2024}",
                    "  url = {" || $url || "}",
                    "  urldate = {" || local:today() || "}"
                ), ",&#10;") || ",&#10;}"
        } catch * {
          'Error: (' || $err:code || '): ' || $err:description
        }
};

(: ~
 : This functions returns the todays date in the form of DD.MM.YYYY.
 :
 : @return A xs:string object
 :)
declare function local:today() as xs:string {
  let $today := fn:current-date()
  return
    format-date($today, "[D01].[M01].[Y0001]")
};
